package adds.sim

import spinal.core._
import spinal.lib._
import spinal.lib.bus.amba4.axis._
import spinal.lib.bus.amba4.axi.Axi4SpecRenamer
import spinal.lib.bus.amba4.axis.Axi4Stream._
import spinal.lib.bus.amba4.axilite._
import spinal.lib.io._
import project.config._
import spinal.core.sim._
import spinal.lib.bus.amba4.axilite.sim._
import adds._
import scala.util.control._

object AddsSim extends App {
  Config.sim.withFstWave.compile(new Adds(16, 16, 12, 12, true, true)).doSim{ dut =>

    dut.clockDomain.reset()
    dut.CfgClockDomain.reset()
    dut.CfgClockDomain.forkStimulus(250)
    dut.clockDomain.forkStimulus(160)
    dut.SimClockDomain.forkStimulus(10)
    dut.clockDomain.waitRisingEdge()
    dut.CfgClockDomain.waitRisingEdge()
    dut.SimClockDomain.waitRisingEdge()
    
    val master = AxiLite4Master(dut.io.Cfg, dut.CfgClockDomain)
    // TONE
    master.reset();
    master.write(0x00, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x04, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x08, List(0xFF.toByte, 0xFF.toByte, 0xFF.toByte, 0x00.toByte));
    master.write(0x0C, List(0x00.toByte, 0x10.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x10, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x01.toByte));
    master.write(0x14, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x80.toByte));
    master.write(0x18, List(0x10.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x00, List(0x02.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    dut.io.AddsOut.ready #= true

    dut.clockDomain.waitRisingEdge();
    dut.clockDomain.waitEdge(100000);

    dut.clockDomain.waitRisingEdge();

    // LFM
    master.reset();
    master.write(0x00, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x04, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x08, List(0xFF.toByte, 0xFF.toByte, 0xFF.toByte, 0x00.toByte));
    master.write(0x0C, List(0x00.toByte, 0x10.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x10, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x01.toByte));
    master.write(0x14, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x80.toByte));
    master.write(0x18, List(0x10.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x00, List(0x02.toByte, 0x01.toByte, 0x00.toByte, 0x00.toByte));
    dut.io.AddsOut.ready #= true
    
    dut.clockDomain.waitRisingEdge();
    dut.clockDomain.waitEdge(100000);

    dut.clockDomain.waitRisingEdge();

    // SFM
    master.reset();
    master.write(0x00, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x04, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x08, List(0xFF.toByte, 0xFF.toByte, 0xFF.toByte, 0x00.toByte));
    master.write(0x0C, List(0x00.toByte, 0x10.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x10, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x01.toByte));
    master.write(0x14, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x80.toByte));
    master.write(0x18, List(0x10.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x00, List(0x02.toByte, 0x02.toByte, 0x00.toByte, 0x00.toByte));
    dut.io.AddsOut.ready #= true
    
    dut.clockDomain.waitRisingEdge();
    dut.clockDomain.waitEdge(100000);

    dut.clockDomain.waitRisingEdge();

    // FMCW
    master.reset();
    master.write(0x00, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x04, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x08, List(0xFF.toByte, 0xFF.toByte, 0xFF.toByte, 0x00.toByte));
    master.write(0x0C, List(0x00.toByte, 0x10.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x10, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x01.toByte));
    master.write(0x14, List(0x00.toByte, 0x00.toByte, 0x00.toByte, 0x80.toByte));
    master.write(0x18, List(0x10.toByte, 0x00.toByte, 0x00.toByte, 0x00.toByte));
    master.write(0x00, List(0x02.toByte, 0x03.toByte, 0x00.toByte, 0x00.toByte));
    dut.io.AddsOut.ready #= true
    
    dut.clockDomain.waitRisingEdge();
    dut.clockDomain.waitEdge(100000);
  }
}